@@ -1,29 +1,25 @@
 package JSON::Path::Evaluator;
-$JSON::Path::Evaluator::VERSION = '0.431';
+$JSON::Path::Evaluator::VERSION = '1.0.4';
 use strict;
 use warnings;
-use 5.008;
 
 # ABSTRACT: A module that recursively evaluates JSONPath expressions with native support for Javascript-style filters
 
 use Carp;
 use Carp::Assert qw(assert);
-use Exporter::Tiny ();
 use JSON::MaybeXS;
 use JSON::Path::Constants qw(:operators :symbols);
 use JSON::Path::Tokenizer qw(tokenize);
-use List::Util qw/pairs/;
+use List::Util qw/pairs uniq/;
 use Readonly;
 use Safe;
 use Scalar::Util qw/looks_like_number blessed refaddr/;
 use Storable qw/dclone/;
-use Sys::Hostname qw/hostname/;
 use Try::Tiny;
 
 # VERSION
-use base q(Exporter);
+use Exporter::Shiny qw/evaluate_jsonpath/;
 our $AUTHORITY = 'cpan:POPEFELIX';
-our @EXPORT_OK = qw/ evaluate_jsonpath /;
 
 Readonly my $OPERATOR_IS_TRUE         => 'IS_TRUE';
 Readonly my $OPERATOR_TYPE_PATH       => 1;
@@ -49,6 +45,7 @@ Readonly my %OPERATORS                => (
     $TOKEN_NOT_EQUAL           => $OPERATOR_TYPE_COMPARISON,    # !=
     $TOKEN_GREATER_EQUAL       => $OPERATOR_TYPE_COMPARISON,    # >=
     $TOKEN_LESS_EQUAL          => $OPERATOR_TYPE_COMPARISON,    # <=
+    $TOKEN_REGEX               => $OPERATOR_TYPE_COMPARISON,    # =~
 );
 
 Readonly my $ASSERT_ENABLE => $ENV{ASSERT_ENABLE};
@@ -75,7 +72,6 @@ sub new {
 
 sub evaluate_jsonpath {
     my ( $json_object, $expression, %args ) = @_;
-
     if ( !ref $json_object ) {
         try {
             $json_object = decode_json($json_object);
@@ -104,6 +100,8 @@ sub evaluate {
     my $json_object = $self->{root};
 
     my $token_stream = [ tokenize($expression) ];
+#use Data::Dumper qw/Dumper/;
+#print Dumper $token_stream;
     shift @{$token_stream} if $token_stream->[0] eq $TOKEN_ROOT;
     shift @{$token_stream} if $token_stream->[0] eq $TOKEN_CHILD;
 
@@ -126,8 +124,8 @@ sub evaluate {
 sub _reftable_walker {
     my ( $self, $json_object, $base_path ) = @_;
 
-    $base_path   = defined $base_path   ? $base_path   : '$';
-    $json_object = defined $json_object ? $json_object : $self->root;
+    $base_path   //= '$';
+    $json_object //= $self->root;
 
     my @entries = ( refaddr $json_object => $base_path );
 
@@ -161,7 +159,7 @@ sub _evaluate {    # This assumes that the token stream is syntactically valid
 
     return unless ref $obj;
 
-    $token_stream = defined $token_stream ? $token_stream : [];
+    $token_stream //= [];
 
     while ( defined( my $token = shift @{$token_stream} ) ) {
         next if $token eq $TOKEN_CURRENT;
@@ -477,33 +475,45 @@ sub _filter_recursive {
 }
 
 sub _process_pseudo_js {
-    my ( $self, $object, $expression ) = @_;
+    my ( $self, $object, $expressions ) = @_;
 
-    my ( $lhs, $operator, $rhs ) = _parse_psuedojs_expression($expression);
+    my @expressions_or = split /\Q||\E/, $expressions;
+    my @matching_or;
 
-    my (@token_stream) = tokenize($lhs);
+    foreach my $expression (@expressions_or) {
+        my @expressions_and = split /\Q&&\E/, $expression;
+        my %matching_and;
 
-    my $index;
+        foreach my $expression (@expressions_and) {
 
-    my @lhs;
-    if ( _hashlike($object) ) {
-        @lhs = map { $self->_evaluate( $_, [@token_stream] ) } values %{$object};
-    }
-    elsif ( _arraylike($object) ) {
-        for my $value ( @{$object} ) {
-            my ($got) = $self->_evaluate( $value, [@token_stream] );
-            push @lhs, $got;
+            my ( $lhs, $operator, $rhs ) = _parse_psuedojs_expression($expression);
+            $lhs =~ s/^\s+|\s+$//g;
+
+            my (@token_stream) = tokenize($lhs);
+
+            if ( _hashlike($object) ) {
+                while (my ($k, $v) = each(%$object)) {
+                  my @got = $self->_evaluate( $v, [@token_stream] );
+                  foreach my $got (@got) {
+                      $matching_and{$k}++ if _compare( $operator, $got, $rhs );
+                  }
+                }
+            }
+            elsif ( _arraylike($object) ) {
+                my $idx = 0;
+                for my $value ( @{$object} ) {
+                    my ($got) = $self->_evaluate( $value, [@token_stream] );
+                    $matching_and{$idx}++ if _compare( $operator, $got, $rhs );
+                    $idx++;
+                }
+            }
+        }
+        while (my ($idx, $val) = each(%matching_and)) {
+            push @matching_or, $idx if ($val == @expressions_and);
         }
     }
 
-    # get indexes that pass compare()
-    my @matching;
-    for ( 0 .. $#lhs ) {
-        my $val = $lhs[$_];
-        push @matching, $_ if _compare( $operator, $val, $rhs );
-    }
-
-    return @matching;
+    return sort(uniq(@matching_or));
 }
 
 sub _parse_psuedojs_expression {
@@ -512,7 +522,7 @@ sub _parse_psuedojs_expression {
 
     my ( $lhs, $operator, $rhs );
 
-    # The operator could be '=', '!=', '==', '===', '<=', or '>='
+    # The operator could be '=', '!=', '==', '===', '<=', '>=' or '=~'
     if ( $expression =~ /$EQUAL_SIGN/ ) {
         my $position = index( $expression, '=' );
         if ( substr( $expression, $position + 1, 1 ) eq $EQUAL_SIGN ) {    # could be '==' or '==='
@@ -523,6 +533,9 @@ sub _parse_psuedojs_expression {
                 $operator = $TOKEN_DOUBLE_EQUAL;
             }
         }
+        elsif ( substr( $expression, $position + 1, 1 ) eq $TILDE_SIGN ) {
+            $operator = $TOKEN_REGEX;
+        }
         else {
             my $preceding_char = substr( $expression, $position - 1, 1 );
             if ( $preceding_char eq $GREATER_THAN_SIGN ) {
@@ -629,6 +642,18 @@ sub _compare {
     if ( $operator eq '!=' || $operator eq '!==' ) {
         return $use_numeric ? ( $lhs != $rhs ) : $lhs ne $rhs;
     }
+
+    if ( $operator eq '=~' ) {
+        if ( $rhs =~ m@^/(.*)/([msian]*)$@ ) {
+            my $regex     = $1;
+            my $modifiers = $2;
+            return $lhs =~ qr/(?$modifiers)$regex/;
+        }
+        else {
+            warn("Regexp need to be sorrounded by //. Allowed modifiers: msian");
+            return 1;
+        }
+    }
     use warnings qw/uninitialized/;
 }
 
@@ -646,7 +671,7 @@ JSON::Path::Evaluator - A module that recursively evaluates JSONPath expressions
 
 =head1 VERSION
 
-version 0.431
+version 1.0.4
 
 =head1 SYNOPSIS
 
@@ -832,7 +857,7 @@ child operator
 
 recursive descent. JSONPath borrows this syntax from E4X.
 
-=item '*' (literal asterisk)
+=item *
 
 wildcard. All objects/elements regardless their names.
 
@@ -881,6 +906,12 @@ Using the JSON in L<SYNOPSIS> above and the JSONPath expression C<$..book[?(@.ca
 the filter expression C<@.category == "fiction"> will match all values having a value of "fiction" for
 the key "category".
 
+Regular expressions are supported using the C<=~> operator, for example:
+C<$..book[?(@.category =~ /Fiction/i)]>.
+This is an extension of the L<Goessner specification|http://goessner.net/articles/JsonPath/>
+introduced by L<Jayway JsonPath|https://github.com/json-path/JsonPath>. Other Jayway operators
+are not currently supported.
+
 =head2 Filtering with Perl
 
 When the script engine is set to "perl", filter
@@ -896,25 +927,21 @@ When filtering in Perl, there are some differences between the JSONPath spec and
 
 =item *
 
-JSONPath uses the token '$' to refer to the root node. As this is not valid Perl, this should be
-
-replaced with '$root' in a filter expression.
+JSONPath uses the token '$' to refer to the root node. As this is not valid Perl, this should be replaced with '$root' in a filter expression.
 
 =item *
 
-JSONPath uses the token '@' to refer to the current node. This is also not valid Perl. Use '$_'
-
-instead.
+JSONPath uses the token '@' to refer to the current node. This is also not valid Perl. Use '$_' instead.
 
 =back
 
 =head1 AUTHOR
 
-Kit Peters <popefelix@gmail.com>
+Aurelia Peters <popefelix@gmail.com>
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is copyright (c) 2021 by Kit Peters.
+This software is copyright (c) 2022 by Aurelia Peters.
 
 This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
